--[[
DHUD4_Pet.lua
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2010 by Horacio Hoyos

This file is part of DHUD4.

    DHUD4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD4.  If not, see <http://www.gnu.org/licenses/>.
--]]
local DHUD4 = LibStub("AceAddon-3.0"):GetAddon("DHUD4");
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD4");
local DogTag = LibStub("LibDogTag-3.0");

local MODNAME = "DHUD4_Pet";
local DHUD4_Pet = DHUD4:NewModule(MODNAME, "AceEvent-3.0");
local VERSION = tonumber(("$Rev: 24 $"):match("%d+"));

local unpack = unpack
local _G = _G
local string_match = string.match
local string_len = string.len

--Fonts
local MAXFONT = 25
local MINFONT = 6

local TEXTURES = {
    {--unHappy
        0.375,0.5625,0,0.359375
    },
    { --normal
        0.1875,0.375,0,0.359375
    },
    { --happy
        0,0.1875,0,0.359375
    }
}

--Local Defaults
local db
local defaults = {
	profile = {
        blizPet = true,
        pet = "c",
		icon = true,
		layout = "HLPR",
		healthBarText = true,
		powerBarText = true,
		healthTextStyle = "[FractionalHP] ([PercentHP:Percent])",
		healthCustomTextStyle = "",
		powerTextStyle = "[FractionalMP] ([PercentMP:Percent])",
		powerCustomTextStyle = "",
		vehicle = true,
		vehBars =  "player",
		barTextSize = 10,
		colors = {
			["0"] = {
                f = {r=0,g=1,b=1},
                h = {r=0,g=0,b=1},
                l = {r=1,g=0,b=1},
            },
            ["1"] = {
                f = {r=1,g=0,b=0},
                h = {r=1,g=0.23,b=0.23},
                l = {r=1,g=0.42,b=0.42},
            },
            ["2"] = {
	            f = {r=0.67,g=0.27,b=0},
	            h = {r=0.67,g=0.27,b=0},
	            l = {r=0.67,g=0.27,b=0}
	        },
            ["3"] = {
                f = {r=1,g=1,b=0},
                h = {r=1,g=1,b=0.23},
                l = {r=1,g=1,b=0.42},
            },
            ["6"] = {
                f = {r=0,g=0.68,b=0.87},
                h = {r=0.23,g=0.71,b=0.84},
                l = {r=0.47,g=0.75,b=0.81},
            },
            ["7"] = {
                f = {r=0,g=1,b=0},
                h = {r=1,g=1,b=0},
                l = {r=1,g=0,b=0},
            },
		},
	}
}


-- Functions required by the configuration options table
local OptGetter, OptSetter, ColorGetter, ColorSetter, SelectGetter, SelectSetter;
do
	local mod = DHUD4_Pet
	function OptGetter(info)
		local key = info[#info]
		return db[key]
	end

	function OptSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end

	function SelectGetter (info, select)
        return db[select]
    end

    function SelectSetter (info, select, value)
       db[select] = value
       mod:Refresh()
    end

    function ColorGetter (info)
        local key1, key2, key3 = info[#info-2], info[#info-1], info[#info]
        return db[key1][key2][key3].r, db[key1][key2][key3].g, db[key1][key2][key3].b
    end

    function ColorSetter (info, r, g, b)
        local key1, key2, key3 = info[#info-2], info[#info-1], info[#info]
        db[key1][key2][key3].r = r
        db[key1][key2][key3].g = g
        db[key1][key2][key3].b = b
        mod:Refresh()
    end
end


--Local Options
local options
local function GetOptions()
	if not options then
        --Options table
    	options = {
    		type = "group",
			name = L["Pet Module"],
			arg = MODNAME,
            order = 3,
			get = OptGetter,
			set = OptSetter,
            childGroups = "tab",
    		args = {
                header = {
                    type = 'description',
                    name = L["The pet module manages pet bars and text and happines icon. The pet bars can track druid mana and vehicle status"],
                    order = 1,
                },
                enabled = {
                    order = 2,
                    type = "toggle",
                    name = L["Enable Pet Module"],
                    get = function() return DHUD4:GetModuleEnabled(MODNAME) end,
                    set = function(info, value) DHUD4:SetModuleEnabled(MODNAME, value) end,
                },
				layout = {
					type = 'group',
					order = 3,
					name = L["Layout"],
					disabled = function() return not DHUD4:GetModuleEnabled(MODNAME); end,
					args = {
                        blizPet = {
                            type ='toggle',
                            order = 1,
                            name = L["Pet Frame"],
                            desc = L["Show Blizzard's pet frame. If you hide the Player's frame no matter your choice here the Pet Frame will be hidden"],
                        },
						pet = {
							type = 'select',
							order = 2,
							name = L["PetBars"],
							desc = L["Place the Pet's bars"],
							set = function (info, value)
									db[info[#info]] = value
									if string_match(value, "c") then
										db.layout = "HLPR";
									else
										db.layout = "HIPO";
									end
									DHUD4_Pet:Refresh();
								end,
							values = DHUD4.layoutType,
						},
						layout = {
							type = 'select',
							order = 3,
							name = L["Bar Layout"],
							values = function ()
									if string_match(db.pet, "c") then
										return { ["HLPR"] = L["Health Left/Power Right"], ["HRPL"] = L["Health Right/Power Left"] }
									else
										return { ["HIPO"] = L["Health Inner/Power Outer"], ["HOPI"] = L["Health Outer/Power Inner"] }
									end
								end,
						},
						icon = {
							type = 'toggle',
							order = 4,
							name = L["Pet Happiness Icon"],
							desc = L["Show/Hide"],
						},
                        vehicle = {
                            type = 'toggle',
                            order = 5,
                            name = L["In Vehicle"],
                            desc = L["Use pet bars to track vehicle/player stats"],
                        },
					},
				},
				barText = DHUD4:StatusBarMenuOptions(4, not DHUD4:GetModuleEnabled(MODNAME), db),
				colors = {
					type = 'group',
					order = 5,
					name = L["Colors"],
                    disabled = function() return not DHUD4:GetModuleEnabled(MODNAME); end,
					get = ColorGetter,
					set = ColorSetter,
					args = {
						["0"] = {
                            type = 'group',
                            name = L["Mana"],
                            args = DHUD4.colorOptions,
                        },
                        ["1"] = {
                            type = 'group',
                            name = L["Rage"],
                            args = DHUD4.colorOptions,
                        },
                        ["2"] = {
                            type = 'group',
                            name = L["Focus (pets)"],
                            args = DHUD4.colorOptions,
                        },
                        ["3"] = {
                            type = 'group',
                            name = L["Energy"],
                            args = DHUD4.colorOptions,
                        },
                        ["6"] = {
                            type = 'group',
                            name = L["Runic Power"],
                            args = DHUD4.colorOptions,
                        },
                        ["7"] = {
                            type = 'group',
                            name = L["Health"],
                            args = DHUD4.colorOptions,
                        },
					},
				},
			},
		}
	end
	return options;
end

-- Local functions & Vars
local ConfigLayout, UpdatePetIcon
local happIcon
local healthBar, powerBar
local druidManaBar, druidTrackMana, druidEclipseBar, druidTrackEclipse
do
	-- Config Bars and Icons
    function ConfigLayout()

        local hbar, hpar, pbar, ppar, iconSide
    	if string_match(db.pet, "c") then
            if string_match(db.layout, "HLPR") then
                hbar, hpar, pbar, ppar = "fl2", DHUD4_LeftFrame, "fr2", DHUD4_RightFrame
                iconSide = "l";
            else
                hbar, hpar, pbar, ppar = "fr2", DHUD4_RightFrame, "fl2", DHUD4_LeftFrame
                iconSide = "r";
            end
        elseif string_match(db.pet, "l") then
            iconSide = "l";
            if string_match(db.layout, "HIPM") then
                hbar, hpar, pbar, ppar = "fl1", DHUD4_LeftFrame, "fl2", DHUD4_LeftFrame
            else
                hbar, hpar, pbar, ppar = "fl2", DHUD4_LeftFrame, "fl1", DHUD4_LeftFrame
            end
        else
            iconSide = "r";
            if string_match(db.layout, "HIPM") then
                hbar, hpar, pbar, ppar = "fr1", DHUD4_RightFrame, "fr2", DHUD4_RightFrame
            else
                hbar, hpar, pbar, ppar = "fr2", DHUD4_RightFrame, "fr1", DHUD4_RightFrame
            end
        end

    	-- Health bar
        healthBar:InitStatusBar(hbar, hpar, 0.5)
        healthBar:PlaceTextures(DHUD4:GetCurrentTexture())
        if db.healthBarText then
            healthBar:InitStatusBarText(hbar, DHUD4:GetPosition(hbar))
        end
        healthBar:SetDisplayOptions(DHUD4:GetDisplayOptions())

        -- Power bar
        powerBar:InitStatusBar(pbar, ppar, 0.5)
        powerBar:PlaceTextures(DHUD4:GetCurrentTexture())
        if db.powerBarText then
            powerBar:InitStatusBarText(pbar, DHUD4:GetPosition(pbar))
        end
        powerBar:SetDisplayOptions(DHUD4:GetDisplayOptions())

        --Happiness Icon
        if ( db.icon and not happIcon) then
            local parent, x = DHUD4_LeftFrame, 40
            if iconSide == "r" then
                x = -x
                parent = DHUD4_RightFrame
            end
            happIcon = CreateFrame("Button", "DHUD4_happ", parent)
            happIcon:ClearAllPoints()
            happIcon:SetPoint("CENTER", parent, "CENTER", x, 12)
            happIcon:SetHeight(20)
            happIcon:SetWidth(20)
            local texture = happIcon:CreateTexture("DHUD4_happ_Texture", "BACKGROUND")
            texture:ClearAllPoints()
			texture:SetAllPoints()
            --texture:SetPoint("TOPLEFT", happIcon, "TOPLEFT", 0, 0)
            --texture:SetPoint("BOTTOMRIGHT", happIcon , "BOTTOMRIGHT", 0, 0)
            texture:SetHeight(20)
            texture:SetWidth(20)
			texture:Show()
            happIcon.texture = texture
        end

    end

	function UpdatePetIcon()

		local happ = GetPetHappiness()
        local health = UnitHealth("pet")
	    if ( (health > 0) and happ ) then
	        local x0, x1, y0, y1 = unpack( TEXTURES[happ] )
			--DHUD4:Debug("UpdatePetIcon", health, happ,x0, x1, y0, y1)
	        happIcon.texture:SetTexture("Interface\\PetPaperDollFrame\\UI-PetHappiness")
	        happIcon.texture:SetTexCoord(x0, x1, y0, y1)
			--happIcon:SetNormalTexture("Interface\\AddOns\\DHUD4\\textures\\DHUD\\auraborder")
			happIcon:Show()
	    else
	        happIcon:Hide()
	    end
	end

end

function DHUD4_Pet:OnInitialize()

    if not self:IsEnabled() then return end
	self.db = DHUD4.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile
	self:SetEnabledState(DHUD4:GetModuleEnabled(MODNAME))
	DHUD4:RegisterModuleOptions(MODNAME, GetOptions, L["Pet"])
    self.layout = false

    -- Bars
    healthBar = DHUD4:CreateStatusBar()
    powerBar = DHUD4:CreateStatusBar()
    DHUD4_Pet.swap = false

end

function DHUD4_Pet:OnEnable()

	--DHUD4:Debug("DHUD4_Pet:OnEnable");
	if not self:IsEnabled() then return end
    self:Refresh()

end

function DHUD4_Pet:OnDisable()

	self:UnregisterAllEvents()
    healthBar:Disable()
    powerBar:Disable()
    self:DisableIcon(happIcon)
end

function DHUD4_Pet:Refresh()

	--DHUD4:Debug(MODNAME, "Refresh");
	db = self.db.profile
	if not self:IsEnabled() then return end

    --BlizzFrames
	if ( not db.blizPet and PetFrame:IsShown() ) then
		PetFrame:UnregisterAllEvents();
	    PetFrame:Hide();
	end

    healthBar:Disable()
    powerBar:Disable()
    self:DisableIcon(happIcon)
    ConfigLayout()
    if (self.layout) then
        self:SetLayout()
    else
        -- Events of interest
        self:RegisterEvent("UNIT_PET")
        self:RegisterEvent("UNIT_HAPPINESS")
        -- Vehicle Swap
        if db.vehicle then
            self:RegisterEvent("UNIT_ENTERED_VEHICLE")
            self:RegisterEvent("UNIT_EXITED_VEHICLE")
        else
            self:UnregisterEvent("UNIT_ENTERED_VEHICLE")
            self:UnregisterEvent("UNIT_EXITED_VEHICLE")
        end
        -- Priority is Pet, Vehicle, Druid
        if ( UnitExists("pet") ) then
            self:UNIT_PET("UNIT_PET", "player")
            self:UNIT_HAPPINESS("UNIT_HAPPINESS", "player")
        elseif ( db.vehicle and UnitInVehicle("player") ) then
            self:UNIT_ENTERED_VEHICLE("UNIT_ENTERED_VEHICLE", "player")
        --[[elseif ( db.druidMana and ( class == "DRUID" ) ) then
            --DHUD4:Debug("Druid mana bar");
            local powerType = UnitPowerType("player")
            if ( powerType ~= 0 ) then
                self:UPDATE_SHAPESHIFT_FORM()
            end--]]
        end
    end
end

function DHUD4_Pet:UNIT_PET(event, who)

	--DHUD4:Debug("Pet", event, who)
	if ((who == "player") and (not UnitInVehicle("player")) ) then
        healthBar:Disable()
        powerBar:Disable()
		if ( UnitExists("pet") ) then
			healthBar:TrackUnitHealth("pet", db.colors[tostring(7)])
            if db.healthBarText then
                healthBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), (string_len(db.healthCustomTextStyle) > 0) and db.healthCustomTextStyle or db.healthTextStyle, "pet")
            end
            powerBar:TrackUnitPower("pet", db.colors[tostring(UnitPowerType("pet"))], UnitPowerType("pet"))
            if db.powerBarText then
                powerBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), (string_len(db.powerCustomTextStyle) > 0) and db.powerCustomTextStyle or db.powerTextStyle, "pet")
            end
        elseif (druidTrackMana) then
            local colors, useText, textTag = DHUD4_Druid:GetManaBarInfo()
            druidManaBar:TrackUnitPower("player", colors, 0)
            if (useText and textTag) then
                druidManaBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), textTag, "player")
            end
         elseif (druidTrackEclipse) then
            local colors, useText, textTag = DHUD4_Druid:GetEclipseBarInfo()
            druidEclipseBar:TrackUnitPower("player", colors, 8)
            if (useText and textTag) then
                druidEclipseBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), textTag, "player")
            end
        end
        return db.icon and UpdatePetIcon()
	end
end

function DHUD4_Pet:UNIT_HAPPINESS(event, who)

    if ( who == "pet" ) then
        UpdatePetIcon()
    end
end

function DHUD4_Pet:UNIT_ENTERED_VEHICLE(event, who)

	--DHUD4:Debug("Pet", event, who)
    if who == "player" then
        if ( UnitInVehicle("player") ) then
			--DHUD4:Debug("Pet, PlayerInVehicle")
			self:UnregisterEvent("UNIT_PET")
			self:UnregisterEvent("UPDATE_SHAPESHIFT_FORM")
			healthBar:Disable()
            powerBar:Disable()
            local unit = self.swap and "player" or "vehicle"
            healthBar:TrackUnitHealth(unit, db.colors[tostring(7)])
            if db.healthBarText then
                healthBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), (string_len(db.healthCustomTextStyle) > 0) and db.healthCustomTextStyle or db.healthTextStyle, unit)
            end
            powerBar:TrackUnitPower(unit, db.colors[tostring(UnitPowerType(unit))], UnitPowerType(unit))
            if db.powerBarText then
                powerBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), (string_len(db.powerCustomTextStyle) > 0) and db.powerCustomTextStyle or db.powerTextStyle, unit)
            end
        end
    end
end

function DHUD4_Pet:UNIT_EXITED_VEHICLE(event, who)

	--DHUD4:Debug("Pet", event, who)
    if ( who == "player" ) then
		self:RegisterEvent("UNIT_PET")
        if (druidShape) then
            self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
        end
        healthBar:Disable(true)
        powerBar:Disable(true)
        -- Priority is Pet, Druid
        if ( UnitExists("pet") ) then
            self:UNIT_PET("UNIT_PET", "player")
        elseif ( db.druidMana and ( class == "DRUID" ) ) then
            local powerType = UnitPowerType("player")
            if ( powerType ~= 0 ) then
                self:UPDATE_SHAPESHIFT_FORM()
            end
        end
    end
end

function DHUD4_Pet:UntrackDruidEclipse(side, colors)
end


function DHUD4_Pet:UNIT_POWER(event, unit, power)

    DHUD4:Debug("Pet UNIT_POWER", unit, power)
    if (unit == "player" and power == "ECLIPSE") then

    end
end

function DHUD4_Pet:DisableIcon(icon)

    if (icon) then
        icon:Hide()
    end
end

function DHUD4_Pet:SwapPet(swap)
    DHUD4_Pet.swap = swap
end


function DHUD4_Pet:SetLayout()

    self.layout = true
    healthBar:Disable()
    powerBar:Disable()
    healthBar:SetLayout("Pet Health", 100, 80)
    powerBar:SetLayout("Pet Power", 100, 80)
    if ( druidManaBar ) then
        if (druidManaBar ~= healthBar and druidManaBar ~= powerBar) then
            druidBar:Disable()
            druidBar:SetLayout("Druid Mana", 100, 50)
        end
    end
    if ( druidEclipseBar ) then
        if (druidEclipseBar ~= healthBar and druidEclipseBar ~= powerBar) then
            druidEclipseBar:Disable()
            druidEclipseBar:SetLayout("Druid Eclipse", 100, 50)
        end
    end
    happIcon.texture:SetTexture("Interface\\PetPaperDollFrame\\UI-PetHappiness")
	happIcon.texture:SetTexCoord(0,0.1875,0,0.359375)
    happIcon:Show()
end


function DHUD4_Pet:EndLayout()

    self.layout = false
    self:Refresh()
end

--[[
    Druid Tracking.
    The Pet bar can track two things: 1 Mana while shapeshifted. 2. Eclipse
    A bar can be picked for each tracking. Mana bar can have 1 color (f,h,l), 
    eclipse bar has 2 colors: 1 for solar 1 for lunar. Shapeshifting events
    and eclipse tracking is done by the Druid module, pet module only offers
    bar control and configuration and keeps track of bars if a vehicle or 
    pet is used.
    
    1. Druid mana is tracked as normal with a power tracking status bar.
    2. Druid eclipse is mode tricky as eclipse power must be tracked. 
--]]

--- Configure the Druid Bars. The module would create the required bars or
-- reference the bars if allready created. The bar can be the same one.
-- @usage DHUD4_Pet:SetupDruidBars(manaBar, eclipseBar)
-- @param manaBar Bar to track mana
-- @param manaBarText True if bar should use text
function DHUD4_Pet:SetupDruidMana(manaBar, manaBarText, colors, textTag)

    if (not druidManaBar or (druidManaBar.frame.bar ~= manaBar)) then
        if (druidManaBar) then
            druidManaBar:Disable()
        end
        druidManaBar.colors = colors
        if ( healthBar.frame.bar == manaBar ) then
            druidManaBar = healthBar
        elseif ( powerBar.frame.bar == manaBar ) then
            druidManaBar = powerBar
        elseif (eclipseBar.frame.bar == manaBar) then
            druidManaBar = druidEclipseBar
        else
            druidManaBar = DHUD4:CreateStatusBar()
            if string_match(manaBar, "r") then
                druidManaBar:InitStatusBar(manaBar, _G["DHUD4_RightFrame"], 0.5)
            else
                druidManaBar:InitStatusBar(manaBar, _G["DHUD4_LeftFrame"], 0.5)
            end
            druidManaBar:PlaceTextures(DHUD4:GetCurrentTexture())
            druidManaBar:SetDisplayOptions(DHUD4:GetDisplayOptions())
        end
    end
    if manaBarText then
        druidManaBar:InitStatusBarText(manaBar, DHUD4:GetPosition(manaBar))
        druidManaBar.textTag = textTag
    else
        druidManaBar.textTag = nil
    end
end

function DHUD4_Pet:SetupDruidEclipse(eclipseBar, eclipseBarText)

    if (not druidEclipseBar or (druidEclipseBar.frame.bar ~= eclipseBar)) then
        if (druidEclipseBar) then
            druidEclipseBar:Disable()
        end
        if ( healthBar.frame.bar == eclipseBar) then
            druidEclipseBar = healthBar
        elseif ( powerBar.frame.bar == eclipseBar) then
            druidEclipseBar = powerBar
        elseif (druidManaBar.frame.bar == eclipseBar) then
            druidEclipseBar =  druidManaBar
        else
            druidEclipseBar = DHUD4:CreateStatusBar()
            if string_match(eclipseBar, "r") then
                druidEclipseBar:InitStatusBar(eclipseBar, _G["DHUD4_RightFrame"], 0.5)
            else
                druidEclipseBar:InitStatusBar(eclipseBar, _G["DHUD4_LeftFrame"], 0.5)
            end
            druidEclipseBar:PlaceTextures(DHUD4:GetCurrentTexture())
            druidEclipseBar:SetDisplayOptions(DHUD4:GetDisplayOptions())
        end
        -- Create Eclipse Icon and Glow animation
        eclipseIcon = CreateFrame("Button", "DHUD4_Eclipse", druidEclipseBar)
        eclipseIcon:SetHeight(23)
        eclipseIcon:SetWidth(23)
        eclipseIcon:ClearAllPoints()
        if string_match(eclipseBar, "r") then
            eclipseIcon:SetPoint("CENTER", druidEclipseBar.frame, "TOP", -5, 12)
        else
            eclipseIcon:SetPoint("CENTER", druidEclipseBar.frame, "TOP", 5, 12)
        end
        local texture = eclipseIcon:CreateTexture("DHUD4_Eclipse_Texture", "BACKGROUND")
        texture:ClearAllPoints()
        texture:SetAllPoints()
        texture:SetHeight(23)
        texture:SetWidth(23)
        texture:Show()
        eclipseIcon.texture = texture
        druidEclipseBar.frame.glow = texture
        local glow = texture:CreateAnimationGroup("pulse")
        glow:SetLooping("repeat")
        local grow = glow:CreateAnimation("Scale", "grow")
        glow:SetOrder(1)
        glow:SetScale(1.08, 1.08)
        glow:SetDuration(0.5)
        glow:SetSmoothing("IN_OUT")
        local shrink =glow: CreateAnimation("Scale", "shrink")
        shrink:SetOrder(2)
        shrink:SetScale(1.08, 1.08)
        shrink:SetDuration(0.5)
        shrink:SetSmoothing("IN_OUT")
        local eclipseAG = druidEclipseBar.frame:CreateAnimationGroup("eclipseActivate")
        local barAlpha =  alphaAG:CreateAnimation("Alpha", "barAlpha")
        barAlpha:SetOrder(1)
        barAlpha:SetDuration(0.6)
        barAlpha:SetChange(1)
        local glowAlpha =  alphaAG:CreateAnimation("Alpha", "glowAlpha")
        glowAlpha:SetOrder(1)
        glowAlpha:SetDuration(0.6)
        glowAlpha:SetChange(1)
        glowAlpha:SetTarget(texture)
        eclipseAG:SetScript("OnFinished", function (self, event)
                self:GetParent():SetAlpha(1)
                self:GetParent().glow:SetAlpha(1)
                self:GetParent().glow.pulse:Play()
            end)
        --druidEclipseBar:CreateAnimationGroup("sunDeactivate")
        --local moonActivateAG = druidEclipseBar.frame:CreateAnimationGroup("moonActivate")
        --druidEclipseBar:CreateAnimationGroup("moonDeactivate")
    end
    if eclipseBarText then
        druidEclipseBar:InitStatusBarText(eclipseBar, DHUD4:GetPosition(eclipseBar))
    end
end

function DHUD4_Pet:TrackDruidMana()

    druidTrackMana = true
    --DHUD4:Debug("Druid mana bar");
    druidManaBar:TrackUnitPower("player", druidManaBar.colors, 0)
    if (druidManaBar.textTag) then
        druidManaBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), druidManaBar.textTag, "player")
    else
        druidManaBar:ConfigBarText()
    end
end

function DHUD4_Pet:UntrackDruidMana()

    if (druidManaBar) then
        druidManaBar:Disable()
    end
    druidTrackMana = false
end

function DHUD4_Pet:TrackDruidEclipse(colors, useText, textTag)

    druidTrackEclipse = true
    --DHUD4:Debug("TrackDruidEclipse")
    druidEclipseBar:TrackUnitPower("player", colors, 8)
    if (useText and textTag) then
        druidEclipseBar:ConfigBarText(DHUD4:GetFont(), db.barTextSize * DHUD4.GetScale(), textTag, "player")
    end
    --
    --eclipseIcon.texture
end


function DHUD4_Pet:UntrackDruidEclipse()

    if (falsedruidEclipseBar) then
        falsedruidEclipseBar:Disable()
    end
    druidTrackEclipse = false

end